//#define USE_WEBSERVER // uncomment this to use WebServer instead of ArduinoHttpServer

#include <WiFi.h>
#include <WiFiClient.h>
#ifdef USE_WEBSERVER
  #include <WebServer.h>
#else
  #include <ArduinoHttpServer.h>
  #include <Arduino.h>
#endif
#include <ESPmDNS.h>

const char *wifiSsid = "My WiFi Network";
const char *wifiPassword = "verysecret";

#ifdef USE_WEBSERVER
  WebServer server(80);
#else
  WiFiServer wifiServer(80);
#endif

const char indexPage[] = "<table>\n"
"	<tr><td>Url:</td><td><input id=\"url\" type=\"text\" value=\"/test\"></td></tr>\n"
"	<tr><td>Interval:</td><td><input id=\"interval\" type=\"text\" value=\"100\"></td></tr>\n"
"	<tr><td><input type=\"button\" value=\"start\" onclick=\"benchmarkStart()\"></td><td><input type=\"button\" value=\"stop\" onclick=\"benchmarkStop()\"></td></tr>\n"
"	<tr><td>Requests:</td><td id=\"v1\"></td></tr>\n"
"	<tr><td>Errors:</td><td id=\"v2\"></td></tr>\n"
"	<tr><td>Time min:</td><td id=\"v3\"></td></tr>\n"
"	<tr><td>Time max:</td><td id=\"v4\"></td></tr>\n"
"	<tr><td>Time avg:</td><td id=\"v5\"></td></tr>\n"
"	<tr><td>Reqs/s:</td><td id=\"v6\"></td></tr>\n"
"</table>\n"
"<script type=\"application/javascript\">\n"
"	'use strict';\n"
"    const $ = document.querySelector.bind(document);\n"
"    const $$ = document.querySelectorAll.bind(document);\n"
"\n"
"    function render() {\n"
"        $('#v1').textContent = window.currentBenchmark.requests;\n"
"        $('#v2').textContent = window.currentBenchmark.errors;\n"
"        $('#v3').textContent = window.currentBenchmark.timeMin;\n"
"        $('#v4').textContent = window.currentBenchmark.timeMax;\n"
"        $('#v5').textContent = Math.round(window.currentBenchmark.timeAvg * 10) / 10;\n"
"        $('#v6').textContent = window.currentBenchmark.requestsPerSecond;\n"
"    }\n"
"\n"
"    function benchmarkStop() {\n"
"        if (window.currentBenchmark?.handle !== undefined) {\n"
"            window.clearInterval(window.currentBenchmark.handle);\n"
"		}\n"
"        window.currentBenchmark = {\n"
"            handle: undefined,\n"
"			url: '',\n"
"            requests: 0,\n"
"            errors: 0,\n"
"            timeMin: Infinity,\n"
"            timeMax: 0,\n"
"            timeAvg: 0,\n"
"			timeAcc: 0,\n"
"			timeAccCount: 0,\n"
"			timeAccBeginTime: 0,\n"
"			requestsPerSecond: 0,\n"
"			interval: 100,\n"
"        }\n"
"        render();\n"
"	}\n"
"\n"
"    async function benchmarkFunc() {\n"
"        const cb = window.currentBenchmark;\n"
"        const beginTime = Date.now();\n"
"        const response = await fetch(cb.url, {method: 'GET'});\n"
"        const duration = Date.now() - beginTime;\n"
"        cb.requests++;\n"
"        if (!response.ok) {\n"
"            cb.errors++;\n"
"		}\n"
"        if (duration < cb.timeMin) {\n"
"            cb.timeMin = duration;\n"
"		}\n"
"        if (duration > cb.timeMax) {\n"
"            cb.timeMax = duration;\n"
"        }\n"
"        cb.timeAcc += duration;\n"
"        cb.timeAccCount++;\n"
"        if (Date.now() - cb.timeAccBeginTime >= 1000) {\n"
"            cb.timeAvg = cb.timeAcc / cb.timeAccCount;\n"
"            cb.requestsPerSecond = cb.timeAccCount;\n"
"            cb.timeAccCount = 0;\n"
"            cb.timeAcc = 0;\n"
"            cb.timeAccBeginTime = Date.now();\n"
"		}\n"
"        render();\n"
"	}\n"
"\n"
"    function benchmarkStart() {\n"
"		benchmarkStop();\n"
"        window.currentBenchmark.url = $('#url').value;\n"
"        window.currentBenchmark.interval = $('#interval').value;\n"
"        window.currentBenchmark.handle = setInterval(benchmarkFunc.bind(window), window.currentBenchmark.interval);\n"
"	}\n"
"\n"
"    benchmarkStop();\n"
"</script>\n";

/*
 * setup function
 */
void setup(void) {
  Serial.begin(115200);

  // Connect to WiFi network
  WiFi.begin(wifiSsid, wifiPassword);
  Serial.println("");

  // Wait for connection
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  /*use mdns for host name resolution*/
  if (!MDNS.begin("esp32benchmark.local")) {
    Serial.println("Error setting up MDNS responder!");
    while (1) {
      delay(1000);
    }
  }
  Serial.println("mDNS responder started");

  #ifdef USE_WEBSERVER
    server.on("/", HTTP_GET, []() {
      server.send(200, "text/html", indexPage);
    });
    server.on("/test", HTTP_GET, []() {
      server.send(200, "text/plain", "OK");
    });
    server.begin();
    const char*headers[] = {"Cookie"};
    server.collectHeaders(headers, 1);
  #else
    wifiServer.begin();
  #endif

  srand(time(NULL));
}

void loop(void) {
  #ifdef USE_WEBSERVER
    server.handleClient();
  #else
    WiFiClient client( wifiServer.available() );
    if (client.connected())
    {
      // Connected to client. Allocate and initialize StreamHttpRequest object.
      ArduinoHttpServer::StreamHttpRequest<1024> httpRequest(client);

      // Parse the request.
      if (httpRequest.readRequest())
      {
         ArduinoHttpServer::Method method( ArduinoHttpServer::Method::Invalid );
         method = httpRequest.getMethod();
 
         if(method == ArduinoHttpServer::Method::Get )
         {
            ArduinoHttpServer::StreamHttpReply httpReply(client, "text/html");            
            httpReply.send(httpRequest.getResource().toString() == "/" ? indexPage : "OK");
         }
      }
   }
  #endif
}